@@ -30,7 +30,12 @@
 #undef LOCAL_DEBUG
 
 static char *link_tmpdir = NULL;
+#ifdef G_OS_WIN32
+static LinkNetIdType use_local_host = LINK_NET_ID_IS_IPADDR;
+#else
 static LinkNetIdType use_local_host = LINK_NET_ID_IS_FQDN;
+#endif
+static const char *fixed_host_net_id = NULL;
 
 /*
  * make_local_tmpdir:
@@ -134,52 +139,91 @@ link_get_tmpdir (void)
 
 #if defined(AF_INET) || defined(AF_INET6) || defined (AF_UNIX)
 
+/* hopefully a sufficiently large default NIC count */
+#ifdef DEFAULT_LINC2_IFNUM
+#undef DEFAULT_LINC2_IFNUM
+#endif
+#define DEFAULT_LINC2_IFNUM (32)
+
 static char *
 get_netid(LinkNetIdType which, 
 	  char *buf, 
 	  size_t len)
 {
-	if (LINK_NET_ID_IS_LOCAL == which)
+	if (LINK_NET_ID_IS_LOCAL == which) {
+#ifndef G_OS_WIN32
 		return strncpy(buf, "localhost", len);
+#else
+		return strncpy(buf, "127.0.0.1", len);
+#endif
+	}
 
-	if (LINK_NET_ID_IS_IPADDR == which) {
+	if ((LINK_NET_ID_IS_IPADDR == which)
+	    || (LINK_NET_ID_IS_CUSTOM == which)) {
 #ifndef G_OS_WIN32
 		struct sockaddr_in *adr = NULL;
-		struct ifreq my_ifreqs[2];
-		struct ifconf my_ifconf;
-		int num, i, sock;
+		struct ifreq *my_ifreqs = NULL;
+		struct ifconf my_ifconf = { 0 };
+		int sock = -1;
+		int num = 0;
+		int i = 0;
 
-		my_ifconf.ifc_len = sizeof(my_ifreqs);
-		my_ifconf.ifc_req = my_ifreqs;
 		sock = socket(AF_INET,SOCK_DGRAM,0);
 		if (-1 == sock) 
 			goto out;
-
+#ifdef SIOCGIFNUM
+		i = ioctl(sock, SIOCGIFNUM, &num);
+		if ((0 > i) || !num) {
+			close(sock);
+			goto out;
+		}
+#else
+		num = DEFAULT_LINC2_IFNUM;
+#endif
+		my_ifconf.ifc_len = sizeof(struct ifreq) * num;
+		my_ifconf.ifc_req = (struct ifreq*)malloc(my_ifconf.ifc_len);
+		if (!my_ifconf.ifc_req) {
+			close(sock);
+			goto out;
+		}
 		if (ioctl(sock,SIOCGIFCONF,&my_ifconf) < 0) {
 			close(sock);
+			free(my_ifconf.ifc_req);
 			goto out;
 		}
 		close(sock);
 
-		num = my_ifconf.ifc_len / sizeof(struct ifreq);
-		if (!num) 
-			goto out;
-
+		my_ifreqs = my_ifconf.ifc_req;
  		for (i = 0; i < num; i++) {
 			adr = (struct sockaddr_in *)&my_ifreqs[i].ifr_ifru.ifru_addr;
-			if (strcmp("127.0.0.1", inet_ntoa(adr->sin_addr)))
+			if (fixed_host_net_id) {
+				if (!strcmp(fixed_host_net_id, inet_ntoa(adr->sin_addr)))
+					break;
+			} else {
+				if (strcmp("127.0.0.1", inet_ntoa(adr->sin_addr)))
+					break;
+			}
+			if (!strcmp("0.0.0.0", inet_ntoa(adr->sin_addr))) {
+				if (i) 
+					adr = (struct sockaddr_in *)&my_ifreqs[--i].ifr_ifru.ifru_addr;
 				break;
+			}
 		}
+		strncpy(buf, (const char*)inet_ntoa(adr->sin_addr), len);
+		free(my_ifconf.ifc_req);
+
 		/* will be 127.0.0.1 anyway, if no other address is defined... */
-		return strncpy(buf, (const char*)inet_ntoa(adr->sin_addr), len);
+		return buf;
 #else
 		SOCKET sock;
 		DWORD nbytes;
 		/* Let's hope 20 interfaces is enough. There doesn't
 		 * seem to be any way to get information about how
 		 * many interfaces there are.
+		 * 
+		 * [Jules] Using the ifnum define instead...
 		 */
-		INTERFACE_INFO interfaces[20]; 
+		INTERFACE_INFO interfaces[DEFAULT_LINC2_IFNUM]; 
 		int i;
 
 		sock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
@@ -200,8 +244,10 @@ get_netid(LinkNetIdType which,
 		for (i = 0; i < nbytes / sizeof (INTERFACE_INFO); i++) {
 			if ((interfaces[i].iiFlags & IFF_UP) &&
 			    !(interfaces[i].iiFlags & IFF_LOOPBACK) &&
-			    interfaces[i].iiAddress.Address.sa_family == AF_INET)
+			    interfaces[i].iiAddress.Address.sa_family == AF_INET) {
+				d_printf("%s:%s:%d: returning %s\n", __FILE__, __FUNCTION__, __LINE__, inet_ntoa(interfaces[i].iiAddress.AddressIn.sin_addr));
 				return strncpy(buf, inet_ntoa(interfaces[i].iiAddress.AddressIn.sin_addr), len);
+			}
 		}
 
 		/* Next look for a loopback IPv4 address */
@@ -218,6 +264,7 @@ get_netid(LinkNetIdType which,
 	}
 
 	if ((LINK_NET_ID_IS_SHORT_HOSTNAME == which) || (LINK_NET_ID_IS_FQDN == which)) {
+                d_printf("%s:%s:%d:gethostname()\n", __FILE__, __FUNCTION__, __LINE__);
 		if (gethostname(buf, len))
 			goto out;
 #ifndef G_OS_WIN32
@@ -249,6 +296,7 @@ get_netid(LinkNetIdType which,
 		struct hostent *he;
 
 		/* gethostbyname() is MT-safe on Windows, btw */
+                d_printf("%s:%s:%d:gethostbyname(%s)\n", __FILE__, __FUNCTION__, __LINE__, buf);
 		he = gethostbyname(buf);
 
 		if (!he)
@@ -282,6 +330,16 @@ link_use_local_hostname (LinkNetIdType use)
         use_local_host = use;
 }
 
+void 
+link_set_local_hostname (const char *host_id)
+{
+	if (!host_id || !strlen(host_id))
+		return;
+
+	fixed_host_net_id = host_id;
+	use_local_host = LINK_NET_ID_IS_CUSTOM;
+}
+
 /*
  * True if succeeded in mapping, else false.
  */
@@ -364,11 +422,27 @@ link_protocol_is_local_ipv46 (const LinkProtocolInfo *proto,
 		if ((af != AF_INET6) && (af != AF_INET))
 			continue;
 
+		/*
+		 * We set the local_addr port number to the same value as
+		 * in the incoming_addr since the port number does not
+		 * affect testing whether the IP address is local or not.
+		 * On some machines, access() fills in the port value so
+		 * saddr will have a non-zero port value, and calling
+		 * getaddrinfo() on "localhost" will have 0 for the port
+		 * value.  So doing this ensures the test will not fail
+		 * if the port numbers are different.  Note that this code
+		 * is protected by the I/O thread so this is handled
+		 * synchronously, so it is safe to modify the static
+		 * local_addr value.
+		 */
 		if (proto->family == AF_INET) {
 			if (af == AF_INET) {
-				if (!memcmp ((struct sockaddr_in *)result->ai_addr,
-					     (struct sockaddr_in *)saddr,
-					     result->ai_addrlen)) {
+				struct sockaddr_in * localaddr;
+				struct sockaddr_in * incomingaddr;
+				localaddr = (struct sockaddr_in *)result->ai_addr;
+				incomingaddr = (struct sockaddr_in *)saddr;
+				localaddr->sin_port = incomingaddr->sin_port;
+				if (!memcmp (localaddr, incomingaddr, result->ai_addrlen)) {
 #ifdef LOCAL_DEBUG
 					g_warning ("local ipv4 address");
 #endif
@@ -378,9 +452,12 @@ link_protocol_is_local_ipv46 (const LinkProtocolInfo *proto,
 		}
 		else {
 			if (af == AF_INET6) {
-				if (!memcmp ((struct sockaddr_in6 *)result->ai_addr,
-					     (struct sockaddr_in6 *)saddr,
-					     result->ai_addrlen)) {
+				struct sockaddr_in6 * localaddr;
+				struct sockaddr_in6 * incomingaddr;
+				localaddr = (struct sockaddr_in6 *)result->ai_addr;
+				incomingaddr = (struct sockaddr_in6 *)saddr;
+				localaddr->sin6_port = incomingaddr->sin6_port;
+				if (!memcmp (localaddr, incomingaddr, result->ai_addrlen)) {
 #ifdef LOCAL_DEBUG
 					g_warning ("local ipv6 address");
 #endif
@@ -396,6 +473,7 @@ link_protocol_is_local_ipv46 (const LinkProtocolInfo *proto,
 #else   /*HAVE_GETADDRINFO*/	
 	if (!local_hostent) {
 		LINK_RESOLV_SET_IPV6;
+                d_printf("%s:%s:%d:gethostbyname(%s)\n", __FILE__, __FUNCTION__, __LINE__, link_get_local_hostname ());
 		local_hostent = gethostbyname (link_get_local_hostname ());
 	}
 
@@ -510,6 +588,7 @@ link_protocol_get_sockaddr_ipv4 (const LinkProtocolInfo *proto,
 			res_init();
 #endif
 		
+                d_printf("%s:%s:%d:gethostbyname(%s)\n", __FILE__, __FUNCTION__, __LINE__, hostname);
 		host = gethostbyname (hostname);
 		if (!host) {
 		  g_free (saddr);
@@ -607,6 +686,7 @@ link_protocol_get_sockaddr_ipv6 (const LinkProtocolInfo *proto,
 #endif
 
 	LINK_RESOLV_SET_IPV6;
+        d_printf("%s:%s:%d:gethostbyname(%s)\n", __FILE__, __FUNCTION__, __LINE__, hostname);
 	host = gethostbyname (hostname);
 	if (!host || host->h_addrtype != AF_INET6) {
 		g_free (saddr);
@@ -787,29 +867,16 @@ link_protocol_get_sockinfo_ipv4 (const LinkProtocolInfo  *proto,
 
 	g_assert (proto && saddr && saddr->sa_family == AF_INET);
 
-	if (sa_in->sin_addr.s_addr != INADDR_ANY) {
+#ifdef G_OS_WIN32
+	hname = inet_ntoa(sa_in->sin_addr);
+#else
+	if (ntohl (sa_in->sin_addr.s_addr) != INADDR_ANY) {
 		host = gethostbyaddr ((char *)&sa_in->sin_addr, 
                                       sizeof (struct in_addr), AF_INET);
 		if (!host)
 			return FALSE;
 		hname = host->h_name;
 	}
-#ifdef G_OS_WIN32
-	/* Make sure looking up that name works */
-	if (strcmp (hname, "localhost") == 0)
-		hname = "127.0.0.1"; /* Don't take any risk */
-	else {
-		char *hname_copy = g_strdup (hname);
-		host = gethostbyname (hname_copy);
-		
-		g_free (hname_copy);
-		if (host == NULL) {
-			/* Nope, use IP address then */
-			hname = inet_ntoa (sa_in->sin_addr);
-		} else {
-			hname = host->h_name;
-		}
-	}
 #endif
 	return link_protocol_get_sockinfo_ipv46 (hname, sa_in->sin_port, 
 						 hostname, portnum);
@@ -1039,6 +1106,22 @@ link_protocol_tcp_setup (int                   fd,
 }
 #endif /* defined(AF_INET) || defined(AF_INET6) */
 
+/*
+ * Comments from bug #422908.
+ * Since version 2.14.9 we now always set the flag
+ * LINK_PROTOCOL_NEEDS_BIND for IPv4 and IPv6.  This was 
+ * changed because on Solaris this bind is needed to open
+ * the ports securely when ORBLocalOnly is being used.  The
+ * maintainers decided it would be best to simply turn the
+ * flag on for all platforms rather than support complicated
+ * ifdefs.  We believe this will not cause any issues.
+ * However, if there is a need to revert, please make sure
+ * that this flag continues to be set for the following 
+ * platforms at minimum:
+ *
+ * IPv4: #if defined (G_OS_WIN32) || defined (__sun)
+ * IPv6: #if defined (__sun)
+ */
 static LinkProtocolInfo static_link_protocols[] = {
 #if defined(AF_INET)
 	{
@@ -1046,12 +1129,7 @@ static LinkProtocolInfo static_link_protocols[] = {
 	AF_INET, 			/* family */
 	sizeof (struct sockaddr_in), 	/* addr_len */
 	IPPROTO_TCP, 			/* stream_proto_num */
-					/* flags */
-#ifdef G_OS_WIN32
-	LINK_PROTOCOL_NEEDS_BIND,
-#else
-	0,
-#endif
+	LINK_PROTOCOL_NEEDS_BIND,       /* flags */
 	link_protocol_tcp_setup, 	/* setup */
 	NULL, 				/* destroy */
 	link_protocol_get_sockaddr_ipv4,/* get_sockaddr */
@@ -1065,7 +1143,7 @@ static LinkProtocolInfo static_link_protocols[] = {
 	AF_INET6, 			/* family */
 	sizeof (struct sockaddr_in6), 	/* addr_len */
 	IPPROTO_TCP, 			/* stream_proto_num */
-	0, 				/* flags */
+        LINK_PROTOCOL_NEEDS_BIND,       /* flags */
 	link_protocol_tcp_setup, 	/* setup */
 	NULL, 				/* destroy */
 	link_protocol_get_sockaddr_ipv6,/* get_sockaddr */
