@@ -38,7 +38,8 @@ giop_connection_real_state_changed (LinkConnection      *cnx,
 void
 giop_connection_close (GIOPConnection *cnx)
 {
-	if (cnx->parent.status == LINK_DISCONNECTED)
+	if ((cnx->parent.status == LINK_DISCONNECTED)
+	    || (cnx->parent.status == LINK_TIMEOUT))
 		return;
 
 	if (cnx->parent.status == LINK_CONNECTED &&
@@ -181,7 +182,7 @@ giop_connection_initiate (gpointer orb_data,
 		link_connection_initiate
 			(giop_connection_get_type (),
 			 proto_name, remote_host_info,
-			 remote_serv_info, options,
+			 remote_serv_info, (LinkConnectionOptions)options,
 			 "orb", orb_data,
 			 "version", (guint) giop_version,
 			 NULL);
